@@ -751,7 +751,7 @@ class InpatientMealOrder (ModelSQL, ModelView):
 
     remarks = fields.Text('Remarks')
 
-    meal_warning = fields.Boolean('Warning',readonly=True,
+    meal_warning = fields.Boolean('Warning',
         help="The patient has special needs on meals")
 
     meal_warning_ack = fields.Boolean('Ack',
@@ -804,7 +804,7 @@ class InpatientMealOrder (ModelSQL, ModelView):
             meal_order.check_health_professional()
             
     def check_meal_order_warning(self):
-        if not self.meal_warning_ack:
+        if not self.meal_warning_ack and self.meal_warning:
             self.raise_user_error('meal_order_warning')
 
     def check_health_professional(self):
@@ -814,19 +814,14 @@ class InpatientMealOrder (ModelSQL, ModelView):
 
     @fields.depends('name')
     def on_change_name(self):
-        meal_warning = False
-        meal_warning_ack = True
         if self.name:
             # Trigger the warning if the patient 
             # has special needs on meals (religion / philosophy )
             if (self.name.patient.vegetarian_type or 
                 self.name.patient.diet_belief):
-                meal_warning = True
-                meal_warning_ack = False
-        return {
-            'meal_warning': meal_warning,
-            'meal_warning_ack': meal_warning_ack,
-        }
+                self.meal_warning = True
+                self.meal_warning_ack = False
+
 
     @classmethod
     def __setup__(cls):
